@@ -15,5 +15,5 @@ from mohou.model.common import (
     ModelT,
     average_float_loss_dict,
 )
+from mohou.model.experimental import ControlModel, MarkoveModelConfig
 from mohou.model.lstm import LSTM, PBLSTM, LSTMConfig, PBLSTMConfig
-from mohou.model.markov import ControlModel, MarkoveModelConfig
