@@ -12,7 +12,12 @@ from mohou.encoder import ImageEncoder
 from mohou.encoding_rule import EncodingRule, EncodingRuleBase
 from mohou.model import LSTM, PBLSTM
 from mohou.model.common import ModelT
-from mohou.model.experimental import DisentangleLSTM, ProportionalModel
+from mohou.model.experimental import (
+    DisentangleLSTM,
+    MarkovModelT,
+    MarkovPredictionModel,
+    ProportionalModel,
+)
 from mohou.trainer import TrainCache
 from mohou.types import ElementDict, RGBImage, TerminateFlag
 from mohou.utils import detect_device
@@ -136,7 +141,7 @@ class LSTMPropagatorBase(PropagatorBase[ModelT]):
         state_with_flag = self.encoding_rule.apply(elem_dict)
         self.fed_state_list.append(state_with_flag)
 
-    def predict(self, n_prop: int) -> List[ElementDict]:
+    def predict(self, n_prop: int, term_threshold: Optional[float] = None) -> List[ElementDict]:
         # prediction
         pred_state_list = self._predict(n_prop)
         elem_dict_list = []
@@ -149,7 +154,9 @@ class LSTMPropagatorBase(PropagatorBase[ModelT]):
 
         return elem_dict_list
 
-    def _predict(self, n_prop: int, force_continue: bool = False) -> List[np.ndarray]:
+    def _predict(
+        self, n_prop: int, force_continue: bool = False, term_threshold: Optional[float] = None
+    ) -> List[np.ndarray]:
         pred_state_list: List[np.ndarray] = []
 
         assert self.static_context is not None, "forgot setting static_context ??"
@@ -168,11 +175,18 @@ class LSTMPropagatorBase(PropagatorBase[ModelT]):
             state_pred_torch = out[0, -1, :]
             state_pred = state_pred_torch.cpu().detach().numpy()
 
+            if term_threshold is None:
+                satisfy_break_condition = False
+            else:
+                satisfy_break_condition = state_pred[-1].item() > term_threshold
+
             if force_continue:
                 state_pred[-1] = CONTINUE_FLAG_VALUE
-
             pred_state_list.append(state_pred)
 
+            if satisfy_break_condition:
+                break
+
         return pred_state_list
 
     def get_device(self) -> Optional[torch.device]:
@@ -283,15 +297,24 @@ class DisentangleLSTMPropagator(LSTMPropagatorBase[DisentangleLSTM]):
         return pred_torch, hidden
 
 
+MarkovPropagatorT = TypeVar("MarkovPropagatorT", bound="MarkovPropagator")
+
+
 @dataclass
-class ProportionalModelPropagator(PropagatorBase[ProportionalModel]):
-    model: ProportionalModel
+class MarkovPropagator(PropagatorBase[MarkovModelT]):
+    model: MarkovModelT
     encoding_rule: EncodingRuleBase
     fed_vector: Optional[np.ndarray] = None
 
     @classmethod
-    def create_default(cls, project_path: Path) -> "ProportionalModelPropagator":
-        tcache = TrainCache.load(project_path, ProportionalModel)
+    @abstractmethod
+    def get_model_type(cls) -> Type[MarkovModelT]:
+        pass
+
+    @classmethod
+    def create_default(cls: Type[MarkovPropagatorT], project_path: Path) -> MarkovPropagatorT:
+        model_type = cls.get_model_type()
+        tcache = TrainCache.load(project_path, model_type)
         encoding_rule = EncodingRule.create_default(project_path)
         return cls(tcache.best_model, encoding_rule)
 
@@ -321,9 +344,24 @@ class ProportionalModelPropagator(PropagatorBase[ProportionalModel]):
         pass
 
 
+@dataclass
+class ProportionalModelPropagator(MarkovPropagator[ProportionalModel]):
+    @classmethod
+    def get_model_type(cls) -> Type[ProportionalModel]:
+        return ProportionalModel
+
+
+@dataclass
+class MarkovPredictionModelPropagator(MarkovPropagator[MarkovPredictionModel]):
+    @classmethod
+    def get_model_type(cls) -> Type[MarkovPredictionModel]:
+        return MarkovPredictionModel
+
+
 class PropagatorSelection(Enum):
     lstm = LSTMPropagator
     pblstm = PBLSTMPropagator
     chimera = ChimeraPropagator
     proportional = ProportionalModelPropagator
+    markov_prediction = MarkovPredictionModelPropagator
     disentangle = DisentangleLSTMPropagator
